
#include <proto/exec.h>
#include <proto/dos.h>
#include <proto/graphics.h>
#include <proto/intuition.h>
#include <dos/dostags.h>
#include <string.h>

#ifndef MEMF_SHARED
#define MEMF_SHARED MEMF_PUBLIC
#endif

const char title[] = "<- close to abort      ";

BOOL aborted = FALSE;

#ifndef __PPC__
__saveds
#endif
void gui (void)

{
struct Process *pr = (struct Process *) FindTask (NULL);
struct Message *msg = NULL;
struct Screen *scr;
struct Window *win;
struct IntuiMessage *mess;
BOOL cont;
ULONG sigs,winsig,prsig;
long w,h;

WaitPort (&pr->pr_MsgPort);
msg = GetMsg (&pr->pr_MsgPort);

msg->mn_Node.ln_Pri = 0;  /* show failure */

if (scr = LockPubScreen (NULL))
	{
	w = TextLength (&scr->RastPort,title,strlen(title)) + 64;
	h = scr->RastPort.TxHeight + scr->WBorTop + 1;

	if (win = OpenWindowTags (NULL,
			WA_Left,scr->MouseX - 16,		/* position window so that the close gadget is under the mouse pointer */
			WA_Top,scr->MouseY - h / 2,
			WA_Width,w,
			WA_Height,h,
			WA_Title,title,
			WA_Flags,WFLG_CLOSEGADGET|WFLG_DRAGBAR|WFLG_DEPTHGADGET|WFLG_ACTIVATE,
			WA_IDCMP,IDCMP_CLOSEWINDOW|IDCMP_VANILLAKEY,
			TAG_END))
		{

		msg->mn_Node.ln_Pri = 1;  /* gui initialised correctly */
		ReplyMsg (msg);
		msg = NULL;

		winsig = 1L << win->UserPort->mp_SigBit;
		prsig  = 1L << pr->pr_MsgPort.mp_SigBit;

/* Do not use DOS functions when using pr_MsgPort for your own purposes ! */

		cont = TRUE;
		do	{
			sigs = Wait (winsig | prsig);

			if (sigs & prsig)
				{
				if (msg = GetMsg (&pr->pr_MsgPort))
					cont = FALSE;			/* got message from main task -> let's quit */
				}

			while (mess = (struct IntuiMessage *) GetMsg (win->UserPort))
				{
				switch (mess->Class)
					{
				case IDCMP_CLOSEWINDOW:
					cont = FALSE;
					break;
				case IDCMP_VANILLAKEY:
					if (mess->Code == 0x1b) /* Esc pressed */
						cont = FALSE;
					break;
					}
				ReplyMsg ((struct Message *) mess);
				}
			}
		while (cont);

		aborted = TRUE;

		if (!msg)							/* main task didn't react yet */
			{
			ModifyIDCMP (win,0);			/* tell intuition that we don't listen to window messages any more */
			SetWindowPointer (win,
				WA_BusyPointer,TRUE,		/* show user that we got his input */
				TAG_END);

			WaitPort (&pr->pr_MsgPort);		/* wait for reaction from main task */
			msg = GetMsg (&pr->pr_MsgPort);
			}

		CloseWindow (win);
		}

	UnlockPubScreen (NULL,scr);
	}

Forbid();								/* make sure we have finished before the reply is sent */
ReplyMsg (msg);							/* tell main task that we have finished */
}


ULONG fib (ULONG n)							/* recursive calculation of fibunacci numbers */

{
if (n == 0) return (0);
if (n == 1) return (1);
if (aborted) return (0);
return (fib(n-1) + fib(n-2));
}


int main (void)

{
struct MsgPort *port;
struct Message *msg;
struct Process *pr;
ULONG i,n,f;

if (msg = AllocVec (sizeof(struct Message),MEMF_CLEAR|MEMF_SHARED))
	{
	if (port = CreateMsgPort())
		{
		if (pr = CreateNewProcTags(
				NP_Entry,gui,
				NP_Name,"fibgui",
				NP_StackSize,10000,
				NP_Priority,1,
				TAG_END))
			{
			msg->mn_ReplyPort = port;
			PutMsg (&pr->pr_MsgPort,msg);	/* establish communication with sub task */
			WaitPort (port);				/* wait for reply from sub task */
			GetMsg (port);

			if (msg->mn_Node.ln_Pri == 1)	/* subtask replied success */
				{
			//	n = 47; /* highest result which fits into an unsigned long int */
				n = 35;

				for (i = 0; !aborted && i <= n; i++)	/* print all fibunacci numbers */
					{
					f = fib(i);
					if (!aborted)
						Printf ("%ld/%ld: %ld\n",i,n,f);
					}

				if (aborted)
					Printf ("*** Break\n");

				msg->mn_ReplyPort = port;
				PutMsg (&pr->pr_MsgPort,msg);	/* tell sub task to quit */
				WaitPort (port);				/* wait for reply from sub task */
				GetMsg (port);
				}
			else
				Printf ("cannot open gui\n");
			}

		DeleteMsgPort (port);
		}
	FreeVec (msg);
	}

return (0);
}
